@@ -219,6 +219,54 @@ eog_full_screen_hide (GtkWidget *widget)
 	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, hide, (widget));
 }
 
+static void 
+show_nth_image (EogFullScreen *fs, unsigned int nth)
+{
+	EogFullScreenPrivate *priv;
+	EogImage *image;
+	EogIter *iter_copy;
+	EogIter *nth_iter;
+	gboolean success = FALSE;
+	
+	priv = fs->priv;
+
+	/* point the current iterator to the nth image of the list */
+	nth_iter = eog_image_list_get_iter_by_pos (priv->list, nth);
+
+	if (nth_iter != NULL) {
+		if (priv->current != NULL) {
+			g_free (priv->current);
+		}
+		priv->current = nth_iter;
+
+		/* view nth image */
+		image = eog_image_list_get_img_by_iter (priv->list, priv->current);
+		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->view), image);
+		priv->switch_time_counter = 0;
+		g_object_unref (image);
+	}
+	else {
+		/* quit diashow */
+		gtk_widget_hide (GTK_WIDGET (fs));
+		return;
+	}
+
+	/* preload next image in current direction */
+	iter_copy = eog_image_list_iter_copy (priv->list, priv->current);
+	if (priv->direction == EOG_DIRECTION_FORWARD) {
+		success = eog_image_list_iter_next (priv->list, iter_copy, TRUE);
+	}
+	else {
+		success = eog_image_list_iter_prev (priv->list, iter_copy, TRUE);
+	}
+
+	if (success) {
+		prepare_load_image (fs, iter_copy);
+	}
+
+	g_free (iter_copy);	
+}
+
 static void
 show_next_image (EogFullScreen *fs)
 {
@@ -313,6 +361,7 @@ eog_full_screen_key_press (GtkWidget *widget, GdkEventKey *event)
 	case GDK_space:
 	case GDK_Right:
 	case GDK_Down:
+	case GDK_Page_Down:
 		if (eog_image_list_length (priv->list) > 1) {
 			priv->direction = EOG_DIRECTION_FORWARD;
 
@@ -329,6 +378,7 @@ eog_full_screen_key_press (GtkWidget *widget, GdkEventKey *event)
 	case GDK_BackSpace:
 	case GDK_Left:
 	case GDK_Up:
+	case GDK_Page_Up:
 		if (eog_image_list_length (priv->list) > 1) {
 			priv->direction = EOG_DIRECTION_BACKWARD;
 
@@ -336,6 +386,19 @@ eog_full_screen_key_press (GtkWidget *widget, GdkEventKey *event)
 			handled = TRUE;
 		}
 		break;
+
+	case GDK_Home:
+		if (eog_image_list_length (priv->list) > 1) {
+			show_nth_image (fs, 0);
+			handled = TRUE;
+		}
+		break;
+
+	case GDK_End:
+		if (eog_image_list_length (priv->list) > 1) {
+			show_nth_image (fs, eog_image_list_length (priv->list) - 1);
+			handled = TRUE;
+		}
 	};
 	
 	if (do_hide) {
@@ -441,14 +504,21 @@ static void
 eog_full_screen_instance_init (EogFullScreen *fs)
 {
 	EogFullScreenPrivate *priv;
+	GdkScreen *screen;
+	GdkRectangle geometry;
+	int monitor;
 
 	priv = g_new0 (EogFullScreenPrivate, 1);
 	fs->priv = priv;
 
+	screen = gtk_window_get_screen (GTK_WINDOW (fs));
+	monitor = gdk_screen_get_monitor_at_window (screen, GTK_WIDGET (fs)->window);
+	gdk_screen_get_monitor_geometry (screen, monitor, &geometry);
+
 	gtk_window_set_default_size (GTK_WINDOW (fs),
-				     gdk_screen_width (),
-				     gdk_screen_height ()); 
-	gtk_window_move (GTK_WINDOW (fs), 0, 0);
+				     geometry.width,
+				     geometry.height);
+	gtk_window_move (GTK_WINDOW (fs), geometry.x, geometry.y);
 
 	priv->loop = TRUE;
 	priv->first_iter = NULL;
@@ -587,7 +657,7 @@ prepare_data (EogFullScreen *fs, EogImageList *image_list, EogImage *start_image
 	}
 	else {
 		priv->direction = EOG_DIRECTION_FORWARD;
-		priv->first_iter = eog_image_list_iter_copy (image_list, priv->current);
+		priv->first_iter = eog_image_list_get_first_iter (image_list);
 		priv->first_image = TRUE;
 		prepare_load_image (fs, priv->current);
 	}
@@ -598,10 +668,11 @@ eog_full_screen_new (EogImageList *image_list, EogImage *start_image)
 {
 	EogFullScreen *fs;
 	EogFullScreenPrivate *priv;
-	GtkWidget     *widget;
+ 	GtkWidget     *widget;
 	GtkStyle      *style;
 	GConfClient   *client;
 	gboolean       upscale = TRUE;
+	gboolean       antialiasing = TRUE;
 
 	g_return_val_if_fail (image_list != NULL, NULL);
 
@@ -639,7 +710,10 @@ eog_full_screen_new (EogImageList *image_list, EogImage *start_image)
 	priv->loop = gconf_client_get_bool (client, EOG_CONF_FULLSCREEN_LOOP, NULL);
 	priv->switch_timeout = gconf_client_get_int (client, EOG_CONF_FULLSCREEN_SECONDS, NULL);
 	upscale = gconf_client_get_bool (client, EOG_CONF_FULLSCREEN_UPSCALE, NULL);
+	antialiasing = gconf_client_get_bool (client, EOG_CONF_VIEW_INTERPOLATE, NULL);
 	eog_scroll_view_set_zoom_upscale (EOG_SCROLL_VIEW (widget), upscale);
+	eog_scroll_view_set_antialiasing (EOG_SCROLL_VIEW (widget), antialiasing);
+	
 	g_object_unref (G_OBJECT (client));
 
 	/* load first image in the background */
