@@ -1,6 +1,6 @@
 /* Eye of Gnome image viewer - main window widget
  *
- * Copyright (C) 2000 The Free Software Foundation
+ * Copyright (C) 2000-2004 The Free Software Foundation
  *
  * Author: Federico Mena-Quintero <federico@gnu.org>
  *         Jens Finke <jens@gnome.org>
@@ -91,6 +91,7 @@ struct _EogWindowPrivate {
 	GtkWidget           *wraplist;
 	GtkWidget           *info_view;
 	GtkWidget           *statusbar;
+	GtkWidget           *n_img_label;
 
 	/* available action groups */
 	GtkActionGroup      *actions_window;
@@ -132,6 +133,7 @@ static gint eog_window_key_press (GtkWidget *widget, GdkEventKey *event);
 static void eog_window_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, 
 				    GtkSelectionData *selection_data, guint info, guint time);
 static void adapt_window_size (EogWindow *window, int width, int height);
+static void update_status_bar (EogWindow *window);
 
 static GtkWindowClass *parent_class;
 
@@ -231,7 +233,7 @@ verb_FileOpen_cb (GtkAction *action, gpointer user_data)
 }
 
 static void
-verb_DirOpen_cb (GtkAction *action, gpointer user_data)
+verb_FolderOpen_cb (GtkAction *action, gpointer user_data)
 {
 	EogWindow *window;
 	EogWindowPrivate *priv;
@@ -249,10 +251,7 @@ verb_DirOpen_cb (GtkAction *action, gpointer user_data)
 	gtk_widget_show_all (dlg);
 	response = gtk_dialog_run (GTK_DIALOG (dlg));
 	if (response == GTK_RESPONSE_OK) {
-		char *folder;
-
-		folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dlg));
-		list = g_slist_append (list, folder);
+		list = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (dlg));
 	}
 
 	gtk_widget_destroy (dlg);
@@ -268,12 +267,6 @@ verb_FileCloseWindow_cb (GtkAction *action, gpointer user_data)
 	eog_window_close (EOG_WINDOW (user_data));
 }
 
-static void
-verb_FileExit_cb (GtkAction *action, gpointer data)
-{
-	eog_window_close_all ();
-}
-
 static void
 verb_EditPreferences_cb (GtkAction *action, gpointer data)
 {
@@ -323,7 +316,7 @@ verb_HelpAbout_cb (GtkAction *action, gpointer data)
 		pixbuf = gdk_pixbuf_new_from_file (EOG_ICONDIR "/gnome-eog.png", NULL);
 
 		about = gnome_about_new (
-			_("Eye of Gnome"),
+			_("Eye of GNOME"),
 			VERSION,
 			"Copyright \xc2\xa9 2000-2004 Free Software Foundation, Inc.",
 			_("The GNOME image viewing and cataloging program."),
@@ -363,7 +356,7 @@ verb_HelpContent_cb (GtkAction *action, gpointer data)
 						 0,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help for Eye of Gnome.\n"
+						 _("Could not display help for Eye of GNOME.\n"
 						   "%s"),
 						 error->message);
 
@@ -518,6 +511,27 @@ verb_ShowHideAnyBar_cb (GtkAction *action, gpointer data)
 		g_object_set (G_OBJECT (priv->statusbar), "visible", visible, NULL);
 		gconf_client_set_bool (priv->client, EOG_CONF_UI_STATUSBAR, visible, NULL);
 	}
+	else if (g_ascii_strcasecmp (gtk_action_get_name (action), "ViewInfo") == 0) {
+		const char *key = NULL;
+		int n_images = 0;
+
+		g_object_set (G_OBJECT (priv->info_view), "visible", visible, NULL);
+
+		if (priv->image_list != NULL) {
+			n_images = eog_image_list_length (priv->image_list);
+		}
+
+		if (n_images == 1) {
+			key = EOG_CONF_UI_INFO_IMAGE;
+		}
+		else if (n_images > 1) {
+			key = EOG_CONF_UI_INFO_COLLECTION;
+		}
+		
+		if (key != NULL) {
+			gconf_client_set_bool (priv->client, key, visible, NULL);
+		}
+	}
 }
 
 /* ================================================
@@ -626,13 +640,11 @@ save_error (SaveData *data)
 
 	/* display generic error dialog, except for FILE_EXISTS error */
 	if (err_code == EOG_IMAGE_ERROR_FILE_EXISTS) {
-		GnomeVFSURI *uri;
-		char *uri_str;
+		char *str;
 
-		uri = eog_image_get_uri (image); /* FIXME: display uri correct */
-		uri_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+		str = eog_image_get_uri_for_display (image); 
 
-		header = g_strdup_printf (_("Overwrite file %s?"), uri_str);
+		header = g_strdup_printf (_("Overwrite file %s?"), str);
 		detail = _("File exists. Do you want to overwrite it?");
 
 		dlg = eog_hig_dialog_new (GTK_WINDOW (data->window), 
@@ -640,8 +652,7 @@ save_error (SaveData *data)
 					  header, detail,
 					  TRUE);
 
-		g_free (uri_str);
-		gnome_vfs_uri_unref (uri);
+		g_free (str);
 
 		gtk_dialog_add_button (GTK_DIALOG (dlg), _("Skip"), EOG_SAVE_RESPONSE_SKIP);
 		gtk_dialog_add_button (GTK_DIALOG (dlg), _("Overwrite"), EOG_SAVE_RESPONSE_OVERWRITE);
@@ -1906,7 +1917,7 @@ eog_window_key_press (GtkWidget *widget, GdkEventKey *event)
 	switch (event->keyval) {
 	case GDK_Q:
 	case GDK_q:
-		verb_FileExit_cb (NULL, NULL);
+		eog_window_close (EOG_WINDOW (widget));
 		break;
 
 	default:
@@ -1997,9 +2008,12 @@ update_ui_visibility (EogWindow *window)
 {
 	EogWindowPrivate *priv;
 	int n_images = 0;
+	gboolean show_info_pane = TRUE;
+
+	g_return_if_fail (EOG_IS_WINDOW (window));
 
 	priv = window->priv;
-	
+
 	if (priv->image_list != NULL) {
 		n_images = eog_image_list_length (priv->image_list);
 	}
@@ -2012,20 +2026,37 @@ update_ui_visibility (EogWindow *window)
 		gtk_action_group_set_sensitive (priv->actions_image, FALSE);
 	}
 	else if (n_images == 1) {
-		/* update window content */
+		/* update window content for single image mode */
+		show_info_pane = gconf_client_get_bool (priv->client, EOG_CONF_UI_INFO_IMAGE, NULL);
+
 		gtk_widget_show_all (priv->vpane);
 		gtk_widget_hide_all (gtk_widget_get_parent (priv->wraplist));
+		if (!show_info_pane) 
+			gtk_widget_hide (priv->info_view);
 
 		gtk_action_group_set_sensitive (priv->actions_window, TRUE);
 		gtk_action_group_set_sensitive (priv->actions_image,  TRUE);
 	}
 	else {
-		/* update window content */
+		/* update window content for collection mode */
+		show_info_pane = gconf_client_get_bool (priv->client, EOG_CONF_UI_INFO_COLLECTION, NULL);
 		gtk_widget_show_all (priv->vpane);
+		if (!show_info_pane)
+			gtk_widget_hide (priv->info_view);
 
 		gtk_action_group_set_sensitive (priv->actions_window, TRUE);
 		gtk_action_group_set_sensitive (priv->actions_image,  TRUE);
 	}
+
+	/* update the toggle menu item for image information pane too */
+	if (n_images > 0) {
+		GtkAction *action;
+
+		action = gtk_ui_manager_get_action (priv->ui_mgr, "/MainMenu/ViewMenu/InfoToggle");
+		g_assert (action != NULL);
+
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), show_info_pane);
+	}
 }
 
 static void 
@@ -2058,6 +2089,8 @@ image_loading_finished_cb (EogImage *image, gpointer data)
 		g_print ("loading finished: %s - (%i|%i)\n", eog_image_get_caption (image), width, height);
 #endif
 	}
+
+	update_status_bar  (window);
 }
 
 static void
@@ -2066,13 +2099,68 @@ image_loading_failed_cb (EogImage *image, const char* message,  gpointer data)
 	g_print ("loading failed: %s\n", eog_image_get_caption (image));
 }
 
+static void
+update_status_bar (EogWindow *window)
+{
+	EogWindowPrivate *priv;
+	int nimg;
+	int nsel;
+	char *n_img_str = NULL;
+	char *str = NULL;
+
+	priv = window->priv;
+
+	/* update number of selected images */	
+	nimg = eog_image_list_length (priv->image_list);
+	if (nimg > 0) {
+		nsel = eog_wrap_list_get_n_selected (EOG_WRAP_LIST (priv->wraplist));
+		/* Images: (n_selected_images) / (n_total_images) */
+		n_img_str = g_strdup_printf ("%i / %i", nsel, nimg);
+	}
+	gtk_label_set_text (GTK_LABEL (priv->n_img_label), n_img_str);
+	
+	if (priv->displayed_image != NULL) {
+		int zoom, width, height;
+		GnomeVFSFileSize bytes = 0;
+
+		zoom = floor (100 * eog_scroll_view_get_zoom (EOG_SCROLL_VIEW (priv->scroll_view)));
+		
+		eog_image_get_size (priv->displayed_image, &width, &height);
+		bytes = eog_image_get_bytes (priv->displayed_image);
+		
+		if ((width > 0) && (height > 0)) {
+			char *size_string;
+
+			size_string = gnome_vfs_format_file_size_for_display (bytes);
+
+			/* [image width] x [image height] pixel  [bytes]    [zoom in percent] */
+			str = g_strdup_printf (_("%i x %i pixel  %s    %i%%"), 
+					       width, height, size_string, zoom);
+			
+			g_free (size_string);
+		}
+	}
+
+	if (str != NULL) {
+		gnome_appbar_set_status (GNOME_APPBAR (priv->statusbar), str);
+		g_free (str);
+	}
+	else {
+		gnome_appbar_set_status (GNOME_APPBAR (priv->statusbar), "");
+	}
+}
+
+static void
+view_zoom_changed_cb (GtkWidget *widget, double zoom, gpointer data)
+{
+	update_status_bar (EOG_WINDOW (data));
+}
+
 static void
 handle_image_selection_changed (EogWrapList *list, EogWindow *window) 
 {
 	EogWindowPrivate *priv;
 	EogImage *image;
-	gchar *str;
-	gint nimg, nsel;
 	char *title = NULL; 
 
 	priv = window->priv;
@@ -2105,14 +2193,8 @@ handle_image_selection_changed (EogWrapList *list, EogWindow *window)
 	
         eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->scroll_view), image);
 	eog_info_view_set_image (EOG_INFO_VIEW (priv->info_view), image);
-	
-	/* update status bar text */	
-	nimg = eog_image_list_length (priv->image_list);
-	nsel = eog_wrap_list_get_n_selected (EOG_WRAP_LIST (priv->wraplist));
-	
-	/* Images: (n_selected_images) / (n_total_images) */
-	str = g_strdup_printf (_("Images: %i/%i"), nsel, nimg);
-	gnome_appbar_set_status (GNOME_APPBAR (priv->statusbar), str);
+
+	update_status_bar (window);
 
 	/* update window title */
 	if (image != NULL) {
@@ -2123,12 +2205,11 @@ handle_image_selection_changed (EogWrapList *list, EogWindow *window)
 		gnome_vfs_uri_unref (uri);
 	}
 	else {
-		title = g_strdup (_("Eye of Gnome"));
+		title = g_strdup (_("Eye of GNOME"));
 	}
 	gtk_window_set_title (GTK_WINDOW (window), title);
 		
 	g_free (title);
-	g_free (str);
 }
 
 typedef struct {
@@ -2196,18 +2277,17 @@ static GtkActionEntry action_entries_window[] = {
   { "HelpMenu", NULL, N_("_Help") },
   { "FileNewWindow",   GTK_STOCK_NEW,   N_("_New"),      "<control>N",  N_("Open a new window"),            G_CALLBACK (verb_FileNewWindow_cb) },
   { "FileOpen",        GTK_STOCK_OPEN,  N_("_Open..."),  "<control>O",  N_("Open a file"),                  G_CALLBACK (verb_FileOpen_cb) },
-  { "FileDirOpen",     GTK_STOCK_OPEN,  N_("Open _Directory..."), "<control><shift>O", N_("Open a directory"), G_CALLBACK (verb_DirOpen_cb) },
+  { "FileFolderOpen",     GTK_STOCK_OPEN,  N_("Open _Folder..."), "<control><shift>O", N_("Open a folder"), G_CALLBACK (verb_FolderOpen_cb) },
   { "FileCloseWindow", GTK_STOCK_CLOSE, N_("_Close"),    "<control>W",  N_("Close window"),                 G_CALLBACK (verb_FileCloseWindow_cb) },
-  { "FileExit",        GTK_STOCK_QUIT,  N_("_Quit"),     "<control>Q",  N_("Close all windows and quit"),   G_CALLBACK (verb_FileExit_cb) },
-  { "EditPreferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL, N_("Preferences for Eye of Gnome"), G_CALLBACK (verb_EditPreferences_cb) },
+  { "EditPreferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL, N_("Preferences for Eye of GNOME"), G_CALLBACK (verb_EditPreferences_cb) },
   { "HelpManual",      GTK_STOCK_HELP,  N_("_Contents"), "F1",          N_("Help On this application"),     G_CALLBACK (verb_HelpContent_cb) },
   { "HelpAbout",       GNOME_STOCK_ABOUT, N_("_About"),	NULL, N_("About this application"),       G_CALLBACK (verb_HelpAbout_cb) }
 };
 
 /* Toggle items */
 static GtkToggleActionEntry toggle_entries_window[] = {
-  { "ViewToolbar",   NULL, N_("_Toolbar"),   NULL, "Change the visibility of the toolbar in the current window",   G_CALLBACK (verb_ShowHideAnyBar_cb), TRUE },
-  { "ViewStatusbar", NULL, N_("_Statusbar"), NULL, "Change the visibility of the statusbar in the current window", G_CALLBACK (verb_ShowHideAnyBar_cb), TRUE }
+  { "ViewToolbar",   NULL, N_("_Toolbar"),   NULL, N_("Changes the visibility of the toolbar in the current window"),   G_CALLBACK (verb_ShowHideAnyBar_cb), TRUE },
+  { "ViewStatusbar", NULL, N_("_Statusbar"), NULL, N_("Changes the visibility of the statusbar in the current window"), G_CALLBACK (verb_ShowHideAnyBar_cb), TRUE },
 };
 
 
@@ -2234,14 +2314,18 @@ static GtkActionEntry action_entries_image[] = {
 };
 
 
+static GtkToggleActionEntry toggle_entries_image[] = {
+  { "ViewInfo",      NULL, N_("Image _Information"), NULL, N_("Change the visibility of the information pane in the current window"), G_CALLBACK (verb_ShowHideAnyBar_cb), TRUE }
+};
+
 /**
  * window_construct:
  * @window: A window widget.
  *
  * Constructs the window widget.
  **/
-static void
-eog_window_construct_ui (EogWindow *window)
+static gboolean
+eog_window_construct_ui (EogWindow *window, GError **error)
 {
 	EogWindowPrivate *priv;
 	gboolean visible;
@@ -2252,9 +2336,10 @@ eog_window_construct_ui (EogWindow *window)
 	GtkWidget *sw;
 	GtkWidget *frame;
         char *filename;
+	guint merge_id;
 
-	g_return_if_fail (window != NULL);
-	g_return_if_fail (EOG_IS_WINDOW (window));
+	g_return_val_if_fail (window != NULL, FALSE);
+	g_return_val_if_fail (EOG_IS_WINDOW (window), FALSE);
 
 	priv = window->priv;
 
@@ -2274,15 +2359,29 @@ eog_window_construct_ui (EogWindow *window)
 	priv->actions_image = gtk_action_group_new ("MenuActionsImage");
 	gtk_action_group_set_translation_domain (priv->actions_image, PACKAGE);
 	gtk_action_group_add_actions (priv->actions_image, action_entries_image, G_N_ELEMENTS (action_entries_image), window);
+	gtk_action_group_add_toggle_actions (priv->actions_image, toggle_entries_image, G_N_ELEMENTS (toggle_entries_image), window);
 	gtk_ui_manager_insert_action_group (priv->ui_mgr, priv->actions_image, 0);
 
-        filename = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_DATADIR, "gnome-2.0/ui/eog-gtk-ui.xml", TRUE, NULL);
-        g_assert (filename);
-	gtk_ui_manager_add_ui_from_file (priv->ui_mgr, filename, NULL);
-        g_free (filename);
+	/* find and setup UI description */
+        filename = gnome_program_locate_file (NULL,
+                                              GNOME_FILE_DOMAIN_APP_DATADIR,
+                                              "eog/eog-gtk-ui.xml",
+                                              FALSE, NULL);
+
+	if (filename == NULL) {
+		g_set_error (error, EOG_WINDOW_ERROR, 
+			     EOG_WINDOW_ERROR_UI_NOT_FOUND,
+			     _("User interface description not found."));
+		return FALSE;
+	}
+
+	merge_id = gtk_ui_manager_add_ui_from_file (priv->ui_mgr, filename, error);
+	g_free (filename);
+	if (merge_id == 0)
+		return FALSE;
 
 	menubar = gtk_ui_manager_get_widget (priv->ui_mgr, "/MainMenu");
-	g_assert (menubar != NULL);
+	g_assert (GTK_IS_WIDGET (menubar));
 	gtk_box_pack_start (GTK_BOX (priv->box), menubar, FALSE, FALSE, 0);
 
 	toolbar = gtk_ui_manager_get_widget (priv->ui_mgr, "/ToolBar");
@@ -2295,7 +2394,7 @@ eog_window_construct_ui (EogWindow *window)
 	priv->recent_model = egg_recent_model_new (EGG_RECENT_MODEL_SORT_MRU);
 	egg_recent_model_set_filter_groups (priv->recent_model, RECENT_FILES_GROUP, NULL);
 	egg_recent_model_set_limit (priv->recent_model, 5);
-
+	
 	recent_widget = gtk_ui_manager_get_widget (priv->ui_mgr, "/MainMenu/FileMenu/EggRecentDocuments");
 	priv->recent_view = egg_recent_view_gtk_new (gtk_widget_get_parent (recent_widget),
 						     recent_widget);
@@ -2307,6 +2406,14 @@ eog_window_construct_ui (EogWindow *window)
 
 	/* add statusbar */
 	priv->statusbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
+	{
+		GtkWidget *frame;
+		frame = g_object_new (GTK_TYPE_FRAME, "shadow-type", GTK_SHADOW_IN, NULL);
+		priv->n_img_label = gtk_label_new ("       ");
+		gtk_container_add (GTK_CONTAINER (frame), priv->n_img_label);
+		gtk_box_pack_start (GTK_BOX (priv->statusbar), frame, FALSE, TRUE, 0);
+	}
+
 	gtk_box_pack_end (GTK_BOX (priv->box), GTK_WIDGET (priv->statusbar),
 			  FALSE, FALSE, 0);
 
@@ -2315,7 +2422,11 @@ eog_window_construct_ui (EogWindow *window)
 
 	/* the image view for the full size image */
  	priv->scroll_view = eog_scroll_view_new ();
-	/* g_object_set (G_OBJECT (priv->scroll_view), "height_request", 250, NULL); */
+	gtk_widget_set_usize (GTK_WIDGET (priv->scroll_view), 100, 100);
+	g_signal_connect (G_OBJECT (priv->scroll_view),
+			  "zoom_changed",
+			  (GCallback) view_zoom_changed_cb, window);
+
 	frame = gtk_widget_new (GTK_TYPE_FRAME, "shadow-type", GTK_SHADOW_IN, NULL);
 	gtk_container_add (GTK_CONTAINER (frame), priv->scroll_view);
 
@@ -2381,6 +2492,7 @@ eog_window_construct_ui (EogWindow *window)
 	g_object_set (G_OBJECT (priv->statusbar), "visible", visible, NULL);
 
 	update_ui_visibility (window);
+	return TRUE;
 }
 
 static void
@@ -2424,14 +2536,15 @@ eog_window_update_properties (EogWindow *window)
  * Return value: A newly-created main window.
  **/
 GtkWidget *
-eog_window_new (void)
+eog_window_new (GError **error)
 {
 	EogWindow *window;
 
-	window = EOG_WINDOW (g_object_new (EOG_TYPE_WINDOW, "title", _("Eye of Gnome"), NULL));
+	window = EOG_WINDOW (g_object_new (EOG_TYPE_WINDOW, "title", _("Eye of GNOME"), NULL));
 
-	eog_window_construct_ui (window);
-	eog_window_update_properties (window);
+	if (eog_window_construct_ui (window, error)) {
+		eog_window_update_properties (window);
+	}
 
 	return GTK_WIDGET (window);
 }
@@ -2574,7 +2687,9 @@ eog_window_open (EogWindow *window, const char *iid, const char *text_uri, GErro
 		g_object_unref (priv->image_list);
 	}
 	priv->image_list = eog_image_list_new ();
-	priv->sig_id_list_prepared = g_signal_connect (G_OBJECT (priv->image_list), "list_prepared", G_CALLBACK (image_list_prepared_cb),
+	priv->sig_id_list_prepared = g_signal_connect (G_OBJECT (priv->image_list), 
+						       "list_prepared", 
+						       G_CALLBACK (image_list_prepared_cb),
 						       window);
 
 	/* fill list with uris */
@@ -2680,7 +2795,7 @@ eog_get_window_list (void)
  * @eog_window: A shell window.
  * 
  * Queries the URI that is being displayed by the specified window.  If the
- * window is not displaying a single image or directory, this will return NULL.
+ * window is not displaying a single image or folder, this will return NULL.
  * 
  * Return value: The URI that is being displayed.
  **/
